### calcsr.cl
### Yosuke Minowa (2008/09/26)

procedure calcsr (input)

file input     {prompt = "Input image"}
char band      {"K", prompt = "Name of Broad band filter", enum="V|R|I|z|J|H|kp|K|Lp|Mp"}
real lambda    {0, prompt = "Wavelength [micron]"}
char pscale    {"20mas", prompt = "Pixel scale [mas]", enum="2.3mas|10mas|20mas|52mas"}
bool mask      {no, prompt = "Fix bad pixel?"}
file bpmask    {prompt = "Name of bad pixel mask frame"}
char psf_fittype {"moffat", prompt = "Profile type to PSF fit", enum="moffat|gaussian"} 
real wgauss    {0.5, prompt = "Sigma of gaussian for object detection"}
int  backsize  {64, prompt = "Default size of the mesh for background subtraction"}
bool savepsf   {no, prompt = "Save ideal psf?"}
file psfimg    {prompt = "Name of ideal psf image"}

begin
	char _input, _bpmask, _pscale, _psfimg
	char tmp_file1, tmp_file2
	char tmp_inputimg, tmp_gaussimg, tmp_checkimg, tmp_bgsubimg
	char tmp_pupilimg, tmp_phaseimg, tmp_psfimg, tmp_psfimg_match
	char tmp_assoc, tmp_conf, tmp_param, tmp_cat, tmp_idealcoo
	char _band
	char _psffit
	int inwidth, inheight, insize
	int n, anum
	int fftsize, fftcenter
	real a
	real _wgauss, _lambda
	real x_obj, y_obj, x_c, y_c
	real r1, r2, r3, r4, r5, r6, r7, r8, r9
	real fluxerr, background
	real flux_obj, peak_obj, fwhm_obj, p2f_obj
	real flux_ideal, peak_ideal, fwhm_ideal, p2f_ideal
	real kron_rad, a_img, b_img, r_obj
	int _backsize
	real ps, ps_tmp
	real fft_dx, fft_dy
	real sr
	real exp1time, coadds, ndr, pixscale
	
	_input = input
	_bpmask = bpmask
	_wgauss = wgauss
	_pscale = pscale
	_band = band
	_lambda = lambda
	_psfimg = psfimg
	_backsize = backsize
	_psffit = psf_fittype
	
	if(!access(_input)){
	     print("Error: Input file is not exist")
	     bye
	}

	# FFT size
	hselect(_input, "NAXIS1", yes) | scan(inwidth)
	hselect(_input, "NAXIS2", yes) | scan(inheight)

	if(inwidth > inheight){
		insize = inwidth
	}else{
		insize = inheight
	}

	n = 1
	a = insize * 0.5
	while(a > 1){
		a = a * 0.5
		n += 1
	}

	fftsize = 2**n

	# Wavelength
	if(_lambda == 0){
	    if(_band == ""){
	        print("Error: Input valid filters name\n")
		bye
	    }else if(_band == "V"){
		_lambda = 0.550
	    }else if(_band == "R"){
		_lambda = 0.660
	    }else if(_band == "I"){
		_lambda = 0.805
	    }else if(_band == "z"){
		_lambda = 1.033
	    }else if(_band == "J"){
		_lambda = 1.25
	    }else if(_band == "H"){
		_lambda = 1.63
	    }else if(_band == "Kp"){
		_lambda = 2.12
	    }else if(_band == "K"){
		_lambda = 2.20
	    }else if(_band == "Lp"){
		_lambda = 3.77
	    }else if(_band == "Mp"){
		_lambda = 4.68
	    }
        }else if(_lambda <= 0){
            print("Error: Input valid wavelength value.\n")
	    bye
        }

	# Pixel scale
	if(_pscale == "2.3mas"){
	    ps = 0.0023392
	}else if(_pscale == "10mas"){
	    ps = 0.010  # IRCS+F-optics pixel scale 
	}else if(_pscale == "20mas"){
	    if(_lambda >= 1.0){
		ps = 0.02041 # IRCS 20 mas pixel scale
	    }else{
	        ps = 0.01988 # HOWFS APOGEE pixel scale
            }
        }else if(_pscale == "52mas"){
	    ps = 0.05275 # IRCS 52mas pixel scale
        }else{
	    print("Error: Input valid pixel scale.\n")
	    bye
	}

	# Initialize display and imexam command
	unlearn("display")
	unlearn("imexam")
	unlearn("rimexam")

	tmp_file1 = mktemp("tmp_file.")
	imexam(_input,1, >> tmp_file1)

	tmp_file2 = mktemp("tmp_file.")
	tail (tmp_file1, nl=2, >> tmp_file2)
	head (tmp_file2, nl=1) | scan(x_obj,y_obj,r1,r2)
	
	del(tmp_file1)
	del(tmp_file2)

	tmp_assoc = mktemp("tmp_assoc.")
	print(x_obj, y_obj, >> tmp_assoc)

	tmp_inputimg = mktemp("tmp_inputimg.")//".fits"
	imcopy(_input, tmp_inputimg, verbose=no)

	if(mask){
	      if(!access(_bpmask)){
		print("Error: Bad pixel mask file is not exist.\n")
		del(tmp_assoc)
		imdel(tmp_inputimg)
		bye
	      }
	      fixpix(tmp_inputimg, _bpmask, linterp=INDEF, cinterp=INDEF, verbose=no, pixels=no)
	      fixpix(tmp_inputimg, _bpmask, linterp=INDEF, cinterp=INDEF, verbose=no, pixels=no)
	      fixpix(tmp_inputimg, _bpmask, linterp=INDEF, cinterp=INDEF, verbose=no, pixels=no)
	}	      

        tmp_gaussimg = mktemp("tmp_gaussimg.")//".fits"
	if(_wgauss > 0){
	      gauss (tmp_inputimg, tmp_gaussimg, _wgauss, ratio=1.,
	      theta=0., nsigma=4., bilinear=yes, boundary="nearest", constant=0.)
	}else{
	      imcopy(tmp_inputimg, tmp_gaussimg, verbose=no)
	}

	if(access("/tmp/tmp_calcsr.sh")){
	      delete("/tmp/tmp_calcsr.sh", verify=no)
        }

	tmp_conf = mktemp("tmp_conf.")
	print("/usr/local/bin/sex -d >", tmp_conf, > "/tmp/tmp_calcsr.sh")
	!sh /tmp/tmp_calcsr.sh	
	print("ASSOC_PARAMS 1,2", >> tmp_conf)
	print("ASSOC_DATA 1,2", >> tmp_conf)
	print("ASSOC_NAME ", tmp_assoc, >> tmp_conf)
	print("ASSOC_RADIUS 5.0", >> tmp_conf)
	print("ASSOC_TYPE NEAREST", >> tmp_conf)
	print("ASSOCSELEC_TYPE MATCHED", >> tmp_conf)

	tmp_param = mktemp("tmp_param.")
	print("X_IMAGE",> tmp_param)
	print("Y_IMAGE",>> tmp_param)
	print("NUMBER_ASSOC", >> tmp_param)
	print("FLUX_BEST",>> tmp_param)
	print("FLUXERR_BEST",>> tmp_param)
	print("BACKGROUND",>> tmp_param)
	print("KRON_RADIUS", >> tmp_param)
	print("A_IMAGE", >> tmp_param)
	print("B_IMAGE", >> tmp_param)

	if(access("/tmp/tmp_calcsr.sh")){
	      delete("/tmp/tmp_calcsr.sh", verify=no)
        }

	tmp_cat = mktemp("tmp_cat.")
	tmp_checkimg = mktemp("tmp_checkimg.")//".fits"
	print("/usr/local/bin/sex -c ", tmp_conf," ",tmp_gaussimg,",",tmp_inputimg,
	" -CATALOG_NAME ", tmp_cat," -CATALOG_TYPE ASCII ",
	" -PARAMETERS_NAME ", tmp_param, " -ANALYSIS_THRESH 1.0 ",
	" -FILTER N ", " -BACK_SIZE ", _backsize, " -BACKPHOTO_TYPE GLOBAL ", " -CHECKIMAGE_TYPE APERTURES ",
	" -CHECKIMAGE_NAME ", tmp_checkimg, " -VERBOSE_TYPE QUIET", > "/tmp/tmp_calcsr.sh") 
	
	!sh /tmp/tmp_calcsr.sh	

	list = tmp_cat
	n = 0
	while(fscan(list, x_c, y_c, anum, flux_obj, fluxerr, background, kron_rad, a_img, b_img) != EOF){
	      n += 1
	}

	if(n != 1){
	     printf("Error: Object detection was failed.\n")
	     del(tmp_param)
	     del(tmp_conf)
	     del(tmp_assoc)
	     del(tmp_cat)
	     imdel(tmp_inputimg)
	     imdel(tmp_gaussimg)
	     imdel(tmp_checkimg)
	     bye
	}
	del(tmp_cat)

	r_obj = kron_rad * sqrt(a_img * b_img)
	if(_backsize < int(2 * r_obj)){
	    _backsize = int(2 * r_obj)	   
	    imdel(tmp_checkimg)

	    if(access("/tmp/tmp_calcsr.sh")){
	      delete("/tmp/tmp_calcsr.sh", verify=no)
            }

	    tmp_cat = mktemp("tmp_cat.")
	    tmp_checkimg = mktemp("tmp_checkimg.")//".fits"
	    print("/usr/local/bin/sex -c ", tmp_conf," ",tmp_gaussimg,",",tmp_inputimg,
	    " -CATALOG_NAME ", tmp_cat," -CATALOG_TYPE ASCII ",
	    " -PARAMETERS_NAME ", tmp_param, " -ANALYSIS_THRESH 1.0 ",
	    " -FILTER N ", " -BACK_SIZE ", _backsize, " -BACKPHOTO_TYPE GLOBAL ", " -CHECKIMAGE_TYPE APERTURES ",
	    " -CHECKIMAGE_NAME ", tmp_checkimg, " -VERBOSE_TYPE QUIET", > "/tmp/tmp_calcsr.sh") 
	
	    !sh /tmp/tmp_calcsr.sh	

	    list = tmp_cat
	    n = 0
	    while(fscan(list, x_c, y_c, anum, flux_obj, fluxerr, background, kron_rad, a_img, b_img) != EOF){
	      n += 1
	    }

	    if(n != 1){
	      printf("Error: Object detection was failed.\n")
	      del(tmp_param)
	      del(tmp_conf)
	      del(tmp_assoc)
	      del(tmp_cat)
	      imdel(tmp_inputimg)
	      imdel(tmp_gaussimg)
	      imdel(tmp_checkimg)
	      bye
	    }
	    del(tmp_cat)   
	}

	tmp_bgsubimg = mktemp("tmp_img.")//".fits"
	imarith(tmp_inputimg,"-",background,tmp_bgsubimg,verbose=no)

	rimexam.backgro = no
	rimexam.fittype = _psffit
	imexam.defkey = "r"
	imexam.imagecu = tmp_assoc
	imexam.use_dis = no
	imexam(tmp_bgsubimg, 1)
	imexam.defkey = "a"
	imexam(tmp_bgsubimg, 1, >> tmp_file1)
	tail (tmp_file1, nl=1) | scan(r1, r2, r3, r4, peak_obj, r5, r6, r7, r8, fwhm_obj, r9)
	del(tmp_file1)

	display(tmp_checkimg, 2)
	imdel(tmp_checkimg)

	p2f_obj = peak_obj / flux_obj
	fwhm_obj = fwhm_obj * ps

	if(_pscale == "10mas" || _pscale == "2.3mas"){
  	    if(access("/tmp/tmp_calcsr.sh")){
	          delete("/tmp/tmp_calcsr.sh", verify=no)
            }
	    
	    tmp_pupilimg = mktemp("tmp_pupilimg.")//".fits"
	    print("mkpupil ",_lambda," ",ps," ",fftsize," ",
	    tmp_pupilimg, > "/tmp/tmp_calcsr.sh")

            fftcenter = int(fftsize * 0.5)
	    fft_dx = x_obj - fftcenter
	    fft_dy = y_obj - fftcenter
	    
	    tmp_phaseimg = mktemp("tmp_phaseimg.")//".fits"
	    print("mkphase ",fft_dx," ",fft_dy," ",fftsize," ",
	    tmp_phaseimg, >> "/tmp/tmp_calcsr.sh")
	    
	    tmp_psfimg = mktemp("tmp_psfimg.")//".fits"
	    print("fft2_shift ",tmp_pupilimg," ",tmp_phaseimg," 1 ",
	    tmp_psfimg, >> "/tmp/tmp_calcsr.sh")

	    !sh /tmp/tmp_calcsr.sh 

	    imdel(tmp_pupilimg)
	    imdel(tmp_phaseimg)
        }else if(_pscale == "20mas"){
	    if(_lambda >= 1.0){
	        if(access("/tmp/tmp_calcsr.sh")){
	              delete("/tmp/tmp_calcsr.sh", verify=no)
                }
	    
	        tmp_pupilimg = mktemp("tmp_pupilimg.")//".fits"
	        print("mkpupil ",_lambda," ",ps," ",fftsize," ",
	        tmp_pupilimg, > "/tmp/tmp_calcsr.sh")
            
                fftcenter = int(fftsize * 0.5)
	        fft_dx = x_obj - fftcenter
	        fft_dy = y_obj - fftcenter
	    
	        tmp_phaseimg = mktemp("tmp_phaseimg.")//".fits"
	        print("mkphase ",fft_dx," ",fft_dy," ",fftsize," ",
	        tmp_phaseimg, >> "/tmp/tmp_calcsr.sh")
	    
	        tmp_psfimg = mktemp("tmp_psfimg.")//".fits"
	        print("fft2_shift ",tmp_pupilimg," ",tmp_phaseimg," 1 ",
	        tmp_psfimg, >> "/tmp/tmp_calcsr.sh")

	        !sh /tmp/tmp_calcsr.sh 

	        imdel(tmp_pupilimg)
	        imdel(tmp_phaseimg)
            }else{
                if(access("/tmp/tmp_calcsr.sh")){
	              delete("/tmp/tmp_calcsr.sh", verify=no)
                }
	    
	        fftsize = fftsize * 2
	        ps_tmp = ps * 0.5

	        tmp_pupilimg = mktemp("tmp_pupilimg.")//".fits"
	        print("mkpupil ",_lambda," ",ps_tmp," ",fftsize," ",
	        tmp_pupilimg, > "/tmp/tmp_calcsr.sh")
            
                fftcenter = int(fftsize * 0.5)
	        fft_dx = x_obj * 2 - fftcenter
	        fft_dy = y_obj * 2 - fftcenter
	    
	        tmp_phaseimg = mktemp("tmp_phaseimg.")//".fits"
	        print("mkphase ",fft_dx," ",fft_dy," ",fftsize," ",
	        tmp_phaseimg, >> "/tmp/tmp_calcsr.sh")
	    
	        tmp_psfimg = mktemp("tmp_psfimg.")//".fits"
	        print("fft2_shift ",tmp_pupilimg," ",tmp_phaseimg," 0.5 ",
	        tmp_psfimg, >> "/tmp/tmp_calcsr.sh")

	        !sh /tmp/tmp_calcsr.sh 

	        imdel(tmp_pupilimg)
	        imdel(tmp_phaseimg)
            } 	    
	}else if(_pscale == "52mas"){
  	    if(access("/tmp/tmp_calcsr.sh")){
	          delete("/tmp/tmp_calcsr.sh", verify=no)
            }
	    
	    fftsize = fftsize * 4
	    ps_tmp = ps * 0.25

	    tmp_pupilimg = mktemp("tmp_pupilimg.")//".fits"
	    print("mkpupil ",_lambda," ",ps_tmp," ",fftsize," ",
	    tmp_pupilimg, > "/tmp/tmp_calcsr.sh")

            fftcenter = int(fftsize * 0.5)
	    fft_dx = x_obj * 4 - fftcenter
	    fft_dy = y_obj * 4 - fftcenter
	    
	    tmp_phaseimg = mktemp("tmp_phaseimg.")//".fits"
	    print("mkphase ",fft_dx," ",fft_dy," ",fftsize," ",
	    tmp_phaseimg, >> "/tmp/tmp_calcsr.sh")
	    
	    tmp_psfimg = mktemp("tmp_psfimg.")//".fits"
	    print("fft2_shift ",tmp_pupilimg," ",tmp_phaseimg," 0.25 ",
	    tmp_psfimg, >> "/tmp/tmp_calcsr.sh")

	    !sh /tmp/tmp_calcsr.sh 

	    imdel(tmp_pupilimg)
	    imdel(tmp_phaseimg)	    	    

	}

        tmp_psfimg_match = mktemp("tmp_psfimg2.")//".fits"
	imcopy(tmp_psfimg//"[1:"//inwidth//","//"1:"//inheight//"]",tmp_psfimg_match,
	verbose=no)
	imdel(tmp_psfimg)

	if(access("/tmp/tmp_calcsr.sh")){
	      delete("/tmp/tmp_calcsr.sh", verify=no)
        }

	tmp_cat = mktemp("tmp_cat.")
	print("/usr/local/bin/sex -c ", tmp_conf," ",tmp_gaussimg,",",tmp_psfimg_match,
        " -CATALOG_NAME ", tmp_cat," -CATALOG_TYPE ASCII ",
        " -PARAMETERS_NAME ", tmp_param, " -ANALYSIS_THRESH 1.0 ",
        " -FILTER N ", " -BACKPHOTO_TYPE GLOBAL ", " -VERBOSE_TYPE QUIET", > "/tmp/tmp_calcsr.sh")

	!sh /tmp/tmp_calcsr.sh	

	list = tmp_cat
	n = 0
	while(fscan(list, x_c, y_c, anum, flux_ideal, fluxerr, background) != EOF){
	      n += 1
	}

	if(n != 1){
	     printf("Error: Object detection was failed.\n")
	     del(tmp_param)
	     del(tmp_conf)
	     del(tmp_assoc)
	     del(tmp_cat)
	     imdel(tmp_inputimg)
	     imdel(tmp_gaussimg)
	     imdel(tmp_bgsubimg)
	     imdel(tmp_psfimg_match)
	     bye
	}
	del(tmp_cat)

	tmp_idealcoo = mktemp("tmp_coo.")
	print(x_c,y_c, > tmp_idealcoo)
	rimexam.backgro = no
	rimexam.fittype = "gaussian"
	imexam.defkey = "a"
	imexam.imagecu = tmp_idealcoo
	imexam.use_dis = no
	if(savepsf){
	    imcopy(tmp_psfimg_match, _psfimg)
	    display(_psfimg, 3)
	}
	imexam(tmp_psfimg_match, 1, >> tmp_file1)
	tail (tmp_file1, nl=1) | scan(r1, r2, r3, r4, peak_ideal, r5, r6, r7, r8, fwhm_ideal, r9)
	del(tmp_file1)
	del(tmp_idealcoo)

	p2f_ideal = peak_ideal / flux_ideal
	fwhm_ideal = fwhm_ideal * ps
	
	sr = p2f_obj / p2f_ideal

	print("")
	print("### Ideal PSF parameter ###")
	print("  Pixel scale = "//_pscale)
	print("  Wavelength = "//_lambda//" [micron]")
	printf("  FWHM = %.3f [arcsec], Peak/Flux = %.5f\n",fwhm_ideal, p2f_ideal)
	print("")
	
	print("### Strehl ratio ###") 
	print("  Image: "//_input)
	printf("  Coordinate: (%.2f, %.2f)\n",x_obj,y_obj)
	printf("  FWHM = %.3f [arcsec], Flux = %.2f [ADU]\n", fwhm_obj, flux_obj)
	printf("  Peak = %.2f [ADU], Peak/Flux = %.5f\n", peak_obj, p2f_obj)
	printf("  Strehl ratio = %.3f\n",sr)
	print("")

	hselect(_input,"EXP1TIME,COADDS,NDR,I_MDFMST",yes) | scan(exp1time,coadds,ndr,pixscale)
	printf("%s %.2f %d %.3f %.3f\n",_input,exp1time,coadds,fwhm_obj, sr)

	del(tmp_param)
	del(tmp_conf)
	del(tmp_assoc)
        imdel(tmp_inputimg)
	imdel(tmp_gaussimg)
	imdel(tmp_bgsubimg)
        imdel(tmp_psfimg_match)

	unlearn("imexam")
	unlearn("rimexam")

end
