## Script for making sky flat frame
## Yosuke Minowa (2008/06/29)

procedure mkskyflat (objlist, output, masklist)

file objlist  {prompt="List of raw object frames with @mark"}
file output   {prompt="Output sky flat frame"}
file masklist {prompt="Output object mask list with @mark"}
bool bpmask   {yes,prompt="Fix bad pixels?"}
file bpfile   {prompt="Name of bad pixel mask frame"}
int  iternum  {50,prompt="Number of clipping iterations"}
int  nclip    {3,prompt="N-sigma rejection limit"}
char conv     {"block 3 3",prompt = "Convolution kernel(objmask)"}
real  hsig     {3,prompt = "Sigma threshold above sky (objmask)"}
real  lsig     {10,prompt = "Sigma threshold below sky (objmask)"}
int  minpixel {50,prompt = "Minimum number of pixels in detected objects(objmask)"}

struct *list1 {mode = 'h'}
struct *list2 {mode = 'h'}
#
#
begin
	char _objlist, _output, _masklist, _bpfile
	char _conv
	real _hsig, _lsig
	int  _minpix
	int nc,itnum
	char tmp_objfile,tmp_maskfile,tmp_nmfile1, tmp_nmfile2, tmp_fflst
	char tmp_nmimg1, tmp_nmimg2, tmp_ffimg, tmp_skyflat
	char fname1,fname2
	real bg_ave

	_objlist = objlist
	_output = output
	_masklist = masklist
	_bpfile = bpfile
	nc = nclip
	itnum = iternum

	_conv = conv 
	_hsig = hsig
	_lsig = lsig
	_minpix = minpixel

	tmp_objfile = mktemp("tmp_file.")
	sections(_objlist,option='fullname',>tmp_objfile)
	
	# normalize raw frames by average
        tmp_nmfile1 = mktemp("tmp_file.")
	list = tmp_objfile

	while(fscan(list,fname1) == 1){
		imstat (fname1, fields="mean", lower=INDEF, upper=INDEF, nclip=itnum, lsigma=nc, usigma=nc, binwidth=0.1, format=no, cache=no) | scan (bg_ave)
		tmp_nmimg1 = mktemp("tmp_img.")
		imarith (fname1,"/",bg_ave,tmp_nmimg1)
		print(tmp_nmimg1,>>tmp_nmfile1)
	}

	# making temporary sky flat frame
	tmp_skyflat = mktemp("tmp_img.")
	zcombine ("@"//tmp_nmfile1, tmp_skyflat, comb="median", rej="sigclip", proj=no, outtyp="real", offset="none", masktyp="none", maskval=0., blan=0., scal="none", weigh="none", zerop="midpt", lthresh=INDEF, hthresh=INDEF, nlo=1., nhig=1., nkee=1., mcli=yes, lsig=3., hsig=3.)
	imdel("@"//tmp_nmfile1)
	del(tmp_nmfile1,ver-,>&"dev$null")

	# temporary flat fielding
	tmp_fflst = mktemp("tmp_file.")
	list = tmp_objfile
	while(fscan(list,fname1) == 1){
		tmp_ffimg = mktemp("tmp_img.")
		imarith(fname1, "/", tmp_skyflat, tmp_ffimg, ver-)
		print(tmp_ffimg,>> tmp_fflst)
	}

	imdel(tmp_skyflat)	

	# making object mask
	if(bpmask){
	   if(access(_bpfile)){
	   	mkobjmask("@"//tmp_fflst, _masklist, bpmexec=yes, bpmask=_bpfile, convolve=_conv, hsigma=_hsig, lsigma=_lsig, minpix=_minpix)
	   }else{
		print("Error: Bad pixel mask file is not exist")
		bye
	   }
        }else{
	   mkobjmask("@"//tmp_fflst, _masklist, bpmexec=no, convolve=_conv, hsigma=_hsig, lsigma=_lsig, minpix=_minpix)
	}
	imdel("@"//tmp_fflst)
	del(tmp_fflst,ver-,>&"dev$null")

	tmp_maskfile = mktemp("tmp_file.")
	sections(_masklist, option='fullname', >tmp_maskfile)

	tmp_nmfile2 = mktemp("tmp_file.")
	
	list1 = tmp_objfile
	list2 = tmp_maskfile

	while(fscan(list1,fname1) == 1){
		if(fscan(list2,fname2) == 1){
			mimstat(fname1, imasks=fname2//"[pl]", omasks="", fields="mean", lower=INDEF, upper=INDEF, nclip=itnum, lsigma=nc, usigma=nc, binwidth=0.1, format=no, cache=no) | scan(bg_ave)
			tmp_nmimg2 = mktemp("tmp_img.")
			imarith (fname1,"/",bg_ave, tmp_nmimg2)
			hedit (tmp_nmimg2, "objmask", fname2//"[pl]", add=yes, verify=no)
			print (tmp_nmimg2,>>tmp_nmfile2)
		}
	}

	zcombine ("@"//tmp_nmfile2, _output, comb="median", rej="sigclip", proj=no, outtyp="real", offset="none", masktyp="!objmask", maskval=0., blan=0., scal="none", weigh="none", zerop="midpt", lthresh=INDEF, hthresh=INDEF, nlo=1., nhig=1., nkee=1., mcli=yes, lsig=3., hsig=3.)

	imdel("@"//tmp_nmfile2)
	del(tmp_objfile,ver-,>&"dev$null")
	del(tmp_maskfile,ver-,>&"dev$null")
	del(tmp_nmfile2,ver-,>&"dev$null")

end
